## @defgroup Library-Plots-Energy 
# RCAIDE/Library/Plots/Energy/__init__.py
# 

"""
RCAIDE Emissions Plotting Package

This module provides visualization tools for analyzing and displaying emissions-related 
data from vehicle and mission simulations. It focuses on greenhouse gas emissions.
"""
# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------

from .plot_CO2e_emissions      import  plot_CO2e_emissions