# RCAIDE/Framework/Analyses/Mission/Segments/Untrimmed/__init__.py
# 

"""
Mission segemtn for analyzing aircraft performance in untrimmed flight conditions, where forces and moments may not be in equilibrium.

See Also
--------
RCAIDE.Library.Mission.Segments.Cruise
RCAIDE.Library.Mission.Segments.Transition
RCAIDE.Library.Mission.Segments.Climb
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------

from . import Untrimmed 