# RCAIDE/Library/Plots/Weights/__init__.py
# 

"""
RCAIDE Weight Analysis Plotting Package

This package contains modules for visualizing aircraft weight breakdowns and 
weight-related analyses.

See Also
--------
RCAIDE.Library.Plots : Parent plotting package
RCAIDE.Library.Analysis.Weights : Weight analysis tools
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------

from .plot_weight_breakdown import plot_weight_breakdown