# RCAIDE/Library/Methods/Powertrain/Converters/Supersonic_Nozzle/__init__.py

"""
This module provides functionality for modeling supersonic nozzles in powertrains. It includes methods for computing 
supersonic nozzle performance and appending supersonic nozzle conditions to simulation results.

See Also
--------
RCAIDE.Library.Methods.Powertrain.Converters
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------
        
from .append_supersonic_nozzle_conditions   import append_supersonic_nozzle_conditions 
from .compute_supersonic_nozzle_performance import compute_supersonic_nozzle_performance