# RCAIDE/Library/Methods/Powertrain/Converters/Expansion_Nozzle/__init__.py

"""
This module provides functionality for modeling expansion nozzles in powertrains. It includes methods for computing 
expansion nozzle performance and appending expansion nozzle conditions to simulation results.

See Also
--------
RCAIDE.Library.Methods.Powertrain.Converters
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------

from .append_expansion_nozzle_conditions    import append_expansion_nozzle_conditions         
from .compute_expansion_nozzle_performance  import compute_expansion_nozzle_performance