# RCAIDE/Library/Methods/Powertrain/Converters/Turbine/__init__.py

"""
This module provides functionality for modeling turbines in powertrains. It includes methods for computing 
turbine performance and appending turbine conditions to simulation results.

See Also
--------
RCAIDE.Library.Methods.Powertrain.Converters
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------

from .append_turbine_conditions   import append_turbine_conditions                     
from .compute_turbine_performance import compute_turbine_performance