# RCAIDE/Library/Methods/Powertrain/Modulators/__init__.py
# 

"""
Collection of methods for analyzing power modulation components in propulsion systems. 
These methods handle devices that control and modify power delivery, such as Electronic 
Speed Controllers (ESCs). The module provides functionality for computing modulator 
performance and managing power flow between energy sources and mechanical converters.

See Also
--------
RCAIDE.Library.Methods.Powertrain.Propulsors
RCAIDE.Library.Components.Powertrain.Modulators
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------
 
from .                        import Electronic_Speed_Controller