# RCAIDE/Methods/Powertrain/Sources/Batteries/Aluminum_Air/__init__.py
# 

"""
This module provides functionality for modeling aluminum-air batteries in powertrains. It includes methods for 
computing aluminum-air cell performance, including aluminum mass and water mass calculations.

See Also
--------
RCAIDE.Library.Methods.Powertrain.Sources.Batteries
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ---------------------------------------------------------------------------------------------------------------------- 

from .compute_al_air_cell_performance import find_aluminum_mass
from .compute_al_air_cell_performance import find_water_mass
