# RCAIDE/Library/Methods/Energy/Fuel_Cells/Common/__init__.py
# 

"""
This module provides a common method for modeling and analyzing fuel cell stacks in aircraft powertrain systems. It includes functionality for computing stack properties based on individual cell characteristics and stack configuration.

See Also
--------
RCAIDE.Library.Components.Powertrain.Converters.Generic_Fuel_Cell_Stack
RCAIDE.Library.Methods.Powertrain.Converters.Fuel_Cells.Proton_Exchange_Membrane
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------

from .compute_stack_properties       import compute_stack_properties