# RCAIDE/Methods/Mission/Segments/Hover/__init__.py
# 

"""
Collection of mission segment types for analyzing vertical flight operations, including hover, 
vertical climb, and descent maneuvers typically associated with VTOL aircraft.

See Also
--------
RCAIDE.Library.Mission.Segments.Transition
RCAIDE.Library.Mission.Segments.Cruise
RCAIDE.Library.Mission.Segments.Ground
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------

from . import Climb
from . import Hover 
from . import Descent