# RCAIDE/Library/Methods/Powertrain/Converters/Generator/__init__.py

# Created:  Heb 2025, M. Clarke 

"""
generator Methods Package

This module contains methods for generator performance analysis and condition handling.
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------

from .compute_generator_performance             import compute_generator_performance 
from .append_generator_conditions               import append_generator_conditions
from .design_optimal_generator                  import design_optimal_generator