# RCAIDE/Library/Methods/Powertrain/Converters/Reformer/__init__.py

# Created:  Jan 2025, M. Clarke, M. Guidotti    

"""
Reformer Methods Package

This module contains methods for modeling fuel reformer performance and operating conditions. 
The methods support both steady-state and transient analysis of catalytic fuel reforming processes.

See Also
--------
RCAIDE.Library.Components.Powertrain.Converters.Reformer
RCAIDE.Library.Methods.Powertrain.Converters.Reformer.compute_reformer_performance
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------

from .compute_reformer_performance    import compute_reformer_performance


